// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

class Empty { };
class Full { };

class Stack {
public:

  Stack() throw();
    // PURPOSE: initializes a Stack
    // REQUIRE: nothing
    // PROMISE: size() will be zero

  unsigned size() const throw();
    // PURPOSE: returns the number of elements on this Stack
    // REQUIRE: nothing
    // PROMISE: return value will be between 0 and 10 inclusive

  int top() const throw(Empty);
    // PURPOSE: returns the top element of this Stack
    // REQUIRE: size() must not be zero
    // PROMISE: nothing

  void push(int elem) throw(Full);
    // PURPOSE: pushes elem onto the end of this Stack
    // REQUIRE: size() must be strictly less than 10
    // PROMISE: size() will increase by 1
    // PROMISE: top() will be the same as parameter elem

  int pop() throw(Empty);
    // PURPOSE: pops and returns the top element from this
    // REQUIRE: size() must not be zero
    // PROMISE: size() will decrease by 1
    // PROMISE: Returns the same as if top() were called at the beginning

protected:
  int data_[10];
  unsigned size_;
};

inline Stack::Stack() throw()
: size_(0)
{ }

inline unsigned Stack::size() const throw()
{ return size_; }

inline int Stack::top() const throw(Empty)
{
  if (size_ == 0)
    throw Empty();
  return data_[size_-1];
}

inline void Stack::push(int elem) throw(Full)
{
  if (size_ == 10)
    throw Full();
  data_[size_++] = elem;
}

inline int Stack::pop() throw(Empty)
{
  if (size_ == 0)
    throw Empty();
  return data_[--size_];
}

int main()
{
  Stack s;
  s.push(42);
  int elem = s.pop();
}
