// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

class Ellipse {
public:
  Ellipse(float width, float height) throw();
    // PROMISE: width() will return the same as parameter width
    // PROMISE: height() will return the same as parameter height
  virtual ~Ellipse() throw();
  virtual void setSize(float width, float height) throw();
    // PROMISE: width() will return the same as parameter width
    // PROMISE: height() will return the same as parameter height
  float width() const throw();
  float height() const throw();
protected:
  float width_, height_;
};

Ellipse::Ellipse(float width, float height) throw()
: width_(width)
, height_(height)
{ }

Ellipse::~Ellipse() throw()
{ }

void Ellipse::setSize(float width, float height) throw()
{
  width_ = width;
  height_ = height;
}

float Ellipse::width() const throw()
{ return width_; }

float Ellipse::height() const throw()
{ return height_; }

void sample(Ellipse& ellipse)
{
  // Set the dimensions of the Ellipse to 10x20
  ellipse.setSize(10, 20);

  // Legitimately rely on the promises made by Ellipse:
  if (ellipse.width() != 10 || ellipse.height() != 20)
    cerr << "Error! Call the hotline at 1-800-BAD-BUGS\n";
}

class Circle : public Ellipse {
public:
  Circle(float initRadius) throw();
    // PROMISE: width() will equal initRadius, height() will equal initRadius

  // But what should we do with setSize(float x, float y); ??
};

Circle::Circle(float initRadius) throw()
: Ellipse(initRadius, initRadius)
{ }

int main()
{
  Circle c(10);
  cout << "initial Circle dimensions = "
       << c.width() << "x" << c.height() << "\n";

  sample(c);
  cout << "final Circle dimensions = "
       << c.width() << "x" << c.height() << '\n';
}
