// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

class Fruit {
public:
  virtual void printClassName() const throw() = 0;
  virtual ~Fruit() throw();
};

Fruit::~Fruit() throw()
{ }

class Apple : public Fruit {
public:
  virtual void printClassName() const throw();
};

void Apple::printClassName() const throw()
{ cout << "Apple\n"; }

class Banana : public Fruit {
public:
  virtual void printClassName() const throw();
};

void Banana::printClassName() const throw()
{ cout << "Banana\n"; }

class Full  { };
class Empty { };

class BagOfFruit {
public:
  BagOfFruit() throw();
  unsigned size() const throw();
  void insert(Fruit& f) throw(Full);
  Fruit& remove() throw(Empty);
protected:
  enum     { maxSize_ = 20 };
  unsigned   size_;
  Fruit*     data_[maxSize_];
};

BagOfFruit::BagOfFruit() throw()
: size_(0) { }

unsigned BagOfFruit::size() const throw()
{ return size_; }

void BagOfFruit::insert(Fruit& f) throw(Full)
{
  if (size_ == maxSize_) throw Full();
  data_[size_++] = &f;
}

Fruit& BagOfFruit::remove() throw(Empty)
{
  if (size_ == 0) throw Empty();
  return *data_[--size_];
}

void insertFruitIntoBag(BagOfFruit& bag, Fruit& fruit)
{
  bag.insert(fruit);
}

class BagOfApple : public BagOfFruit {
public:
  BagOfApple() throw();
  void insert(Apple& a) throw(Full);
  Apple& remove() throw(Empty);
};

BagOfApple::BagOfApple() throw()
: BagOfFruit()
{ }

void BagOfApple::insert(Apple& a) throw(Full)
{ BagOfFruit::insert(a); }

Apple& BagOfApple::remove() throw(Empty)
{ return (Apple&) BagOfFruit::remove(); }

int main()
{
  BagOfApple bagOfApple;
  Banana banana;
  insertFruitIntoBag(bagOfApple, banana);

  cout << "Removing an Apple from bagOfApple: ";
  Apple& a2 = bagOfApple.remove();
  a2.printClassName();
}
