// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <cstdlib>
#include <iostream>
using namespace std;

class Car {
public:

  Car() throw();
  Car(const Car& c) throw();

  // No need for an explicit assignment operator or destructor
  // since these don't create new Car objects.

  static int num() throw();      // Class service
  int odometer() const throw();  // Object service
  void drive() throw();          // Object service: you drive a Car, not a factory

private:
  static int num_;    // Class data
  int        miles_;  // Object data
};

Car::Car() throw()
: miles_(0)
{
  cout << "Car ctor\n";
  ++num_;
}

Car::Car(const Car& c) throw()
: miles_(c.miles_)
{
  cout << "Car copy\n";
  ++num_;
}

int Car::num() throw()   // Should be in same source file as num_; see FAQ 16.04
{ return num_; }

int Car::odometer() const throw()
{ return miles_; }

void Car::drive() throw()
{ ++miles_; }

int Car::num_ = 0;  // Class data is automatically initialized, often before main()

void fiddleWithObject(Car& car) throw()
{
  while (rand() % 10 != 0)
    car.drive();
  cout << "car.odometer() = " << car.odometer() << '\n';
}

void fiddleWithClass() throw()
{
  cout << "Car::num() returns " << Car::num() << '\n';

  #ifdef GENERATE_ERROR
    Car::drive();    // ERROR: can't drive a factory
    Car::odometer(); // ERROR: factories don't have odometers
  #endif
}

int main()
{
  fiddleWithClass();

  {
    Car a;
    fiddleWithClass();
    fiddleWithObject(a);
  }

  fiddleWithClass();
}
