// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <new>
#include <cstring>
using namespace std;

class MyString {
public:
  MyString(const char* s) throw(bad_alloc);   // ctor to promote a const char*
  MyString(const MyString& source) throw(bad_alloc);        // copy constructor
  ~MyString() throw();                                      // destructor
  MyString& operator= (const MyString& s) throw(bad_alloc); // assignment operator
protected:
  unsigned len_;   // ORDER DEPENDENCY; see FAQ 22.10
  char*    data_;
};

MyString::MyString(const char* s) throw(bad_alloc)
: len_(strlen(s))
, data_(new char[len_+1])
{ memcpy(data_, s, len_+1); }

MyString::MyString(const MyString& source) throw(bad_alloc)
: len_(source.len_)
, data_(new char[source.len_+1])
{ memcpy(data_, source.data_, len_+1); }

MyString::~MyString() throw()
{ delete[] data_; }

int main()
{
  MyString a = "xyzzy"; //calls MyString::MyString(const char*)
  MyString b = a;       //calls MyString::MyString(const MyString&)
}
