// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <stdexcept>
using namespace std;

class Fred { };

class Array {
public:
  Fred&       operator[] (unsigned i)       throw(out_of_range);
  const Fred& operator[] (unsigned i) const throw(out_of_range);
protected:
  enum { capacity_ = 100 };
  Fred data_[capacity_];
};

Fred& Array::operator[] (unsigned i) throw(out_of_range)
{
  if (i >= capacity_) throw out_of_range("Array::operator[]");
  return data_[i];
}

const Fred& Array::operator[] (unsigned i) const throw(out_of_range)
{
  if (i >= capacity_) throw out_of_range("Array::operator[] const");
  return data_[i];
}

void sample(Array& a, const Array& b)
{
  Fred x, y;
  x = a[3];    // Okay: get a[3]
  a[3] = y;    // Okay: change a[3]

  x = b[3];    // Okay: get b[3]
  #ifdef GENERATE_ERROR
    b[3] = y;  // Error: b[3] cannot be changed
  #endif
}

int main()
{
  Array a, b;
  sample(a, b);
}
