// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <cstring>
#include <new>
using namespace std;

template<class T>
class Array {
public:
  Array(unsigned len=10);
  Array(const Array<T>& a);                 // copy constructor
  Array<T>& operator= (const Array<T>& a);  // assignment
 ~Array() throw();
  unsigned len() const throw();
  const T& operator[] (unsigned i) const throw();
  T&       operator[] (unsigned i)       throw();
protected:
  unsigned  len_;
  T*        arr_;
};

template<class T> inline
Array<T>::Array(unsigned len)
: len_(len), arr_(new T[len]) { }

template<class T> inline
Array<T>::~Array() throw()
{ delete[] arr_;  }

template<class T> inline
unsigned Array<T>::len() const throw()
{ return len_; }

template<class T> inline
const T& Array<T>::operator[] (unsigned i) const throw()
{ return arr_[i]; }

template<class T> inline
T& Array<T>::operator[] (unsigned i) throw()
{ return arr_[i]; }

template<class T>
Array<T>::Array(const Array<T>& a)
: len_(a.len_)
, arr_(new T[a.len_])
{
  #if 1
    for (unsigned i = 0; i < len_; ++i)    // Good: the T objects copy themselves
      arr_[i] = a.arr_[i];
  #else
    memcpy(arr_, a.arr_, len_*sizeof(T));  // Bad: manipulates the T objects
  #endif
}

template<class T>
Array<T>& Array<T>::operator= (const Array<T>& a)
{
  if (len_ != a.len_) {                    // Makes self-assignment harmless; see FAQ ????
    T* arr2 = new T[a.len_];
    delete[] arr_;
    arr_ = arr2;
    len_ = a.len_;
  }

  #if 1
    // GOOD FORM: lets the T objects copy themselves:
    for (unsigned i = 0; i < len_; ++i)    // Good: the T objects copy themselves
      arr_[i] = a.arr_[i];
  #else
    // BAD FORM: manipulates the bits of the T objects:
    memcpy(arr_, a.arr_, len_*sizeof(T));  // Bad: manipulates the T objects
  #endif

  return *this;
}
