// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <cstdlib>
#include <iostream>
using namespace std;

class X { };
class Y { };
class X2 : public X { };

class Base {
public:
  virtual void f() throw(X);
    //PROMISE: may throw 'X', but never throws anything else
  virtual ~Base() throw();
};

Base::~Base() throw()
{ }

void userCode(Base& base) throw()
{
  try {
    base.f();
    cout << "OK: base.f() didn't throw anything\n";
  }
  catch (X& x) {
    cout << "OK: base.f() threw an X\n";
  }
  catch (...) {
    cout << "huh? base.f() threw something other than X!\n";
  }
}

class Derived : public Base {
public:
  virtual void f() throw(X, X2, Y);
    //PROMISE:
    //  may throw X   // ok: users will be expecting this
    //  may throw X2  // ok: X2 is derived from X
    //  may throw Y   // Bad form: violates the promise of Base
};

void Base::f() throw(X)
{ if (rand() % 2 == 0) throw X(); }

void Derived::f() throw(X, X2, Y)
{
  int r = rand() % 4;
  if (r == 0) throw X();   // ok: Base users are expecting X
  if (r == 1) throw X2();  // ok: Base users are expecting X
  if (r == 2) throw Y();   // Bad: Base users don't expect Y
}

int main()
{
  Base    b;
  Derived d;
  cout << "using 'Base'\n";
  for (int i = 0; i < 10; ++i)
    userCode(b);
  cout << "using 'Derived'\n";
  for (int j = 0; j < 10; ++j)
    userCode(d);
}
