// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

// pretend this is the escape character
const char* const esc = "ESC";

enum Type { EPSON, PROPRINTER, STAR };

class Printer {
public:
  virtual ~Printer() throw();
  virtual Type type() const throw() = 0;
};

Printer::~Printer() throw()
{ }

class EpsonPrinter : public Printer {
public:
  virtual Type type() const throw();
  void italicsEpson(const char* s) throw();
};

Type EpsonPrinter::type() const throw()
{ return EPSON; }

void EpsonPrinter::italicsEpson(const char* s) throw()
{ cout << esc << "i+" << s << esc << "i-"; }

class ProprinterPrinter : public Printer {
public:
  virtual Type type() const throw();
  void italicsProprinter(const char* s) throw();
};

Type ProprinterPrinter::type() const throw()
{ return PROPRINTER; }

void ProprinterPrinter::italicsProprinter(const char* s) throw()
{ cout << esc << "[i" << s << esc << "[n"; }

class StarPrinter : public Printer {
public:
  virtual Type type() const throw();
  void italicsStar(const char* s) throw();
};

Type StarPrinter::type() const throw()
{ return STAR; }

void StarPrinter::italicsStar(const char* s) throw()
{ cout << esc << "x" << s << esc << "y"; }

void printUsingItalics(Printer& p, const char* s) throw()
{
  switch (p.type()) {
    case EPSON:
      ((EpsonPrinter&) p).italicsEpson(s);
      break;
    case PROPRINTER:
      ((ProprinterPrinter&) p).italicsProprinter(s);
      break;
    case STAR:
      ((StarPrinter&) p).italicsStar(s);
      break;
    default:
      cerr << "Call company representative at 1-800-BAD-BUGS\n";
  }
}
