// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

class Asset {
public:
  virtual ~Asset() throw();
  virtual bool isLiquidatable() const throw();
};

Asset::~Asset() throw()
{ }

bool Asset::isLiquidatable() const throw()
{ return false; }

class LiquidAsset : public Asset {
public:
  LiquidAsset(int value=100) throw();
  int  getValue() const throw();
  void setValue(int value) throw();
  virtual bool isLiquidatable() const throw();
protected:
  int value_;    //value of this asset
};

LiquidAsset::LiquidAsset(int value) throw()      : value_(value) { }
int LiquidAsset::getValue() const throw()        { return value_;  }
void LiquidAsset::setValue(int value) throw()    { value_ = value; }
bool LiquidAsset::isLiquidatable() const throw() { return true;    }

int sample(Asset& asset) throw()
{
  int value;
  if (asset.isLiquidatable()) {
    LiquidAsset& liquidAsset = (LiquidAsset&) asset;
    value = liquidAsset.getValue();
    liquidAsset.setValue(0);
    cout << "Liquidated $" << value << '\n';
  } else {
    value = 0;
    cout << "Sorry, couldn't liquidate this asset\n";
  }
  return value;
}

int main()
{
  Asset       a;
  LiquidAsset b;

  sample(a);
  sample(b);
}
