// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

class Base {
public:
  virtual ~Base()         throw();
  virtual void f(float x) throw();
  virtual void g(float x) throw();
          void h(float x) throw();
};

Base::~Base() throw()
  { }
void Base::f(float x) throw()
  { cout << "Base::f(float)\n"; }
void Base::g(float x) throw()
  { cout << "Base::g(float)\n"; }
void Base::h(float x) throw()
  { cout << "Base::h(float)\n"; }

class Derived : public Base {
public:
  virtual void f(float x) throw();  // Good: overrides Base::f(float)
  virtual void g(int x)   throw();  // Bad: hides Base::g(float)
          void h(float x) throw();  // Bad: redefines a non-virtual function
};

void Derived::f(float x) throw()
  { cout << "Derived::f(float)\n"; }
void Derived::g(int x) throw()
  { cout << "Derived::g(int)\n"; }
void Derived::h(float x) throw()
  { cout << "Derived::h(float)\n"; }

void sampleOne(Base& b, Derived& d)
{
  b.f(3.14f);  // Good: if the object is a Derived, calls Derived::f(float)
  d.f(3.14f);
}

int main()
{
  Derived d;
  sampleOne(d, d);
  return 0;
}

void sampleTwo(Base& b, Derived& d)
{
  b.g(3.14f);
  d.g(3.14f);  // Bad: converts 3.14 to 3 and calls Derived::g(int)
}

int mainTwo()
{
  Derived d;
  sampleTwo(d, d);
  return 0;
}

void sampleThree(Base& b, Derived& d)
{
  b.h(3.14f);  // Bad: calls Base::h(float) - does not use dynamic binding
  d.h(3.14f);
}

int mainThree()
{
  Derived d;
  sampleThree(d, d);
  return 0;
}
