// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <iostream>
using namespace std;

class Base {
public:
  virtual ~Base() throw();
  void f(int x)   throw();
  void f(float x) throw();
protected:
  virtual void f_i(int x)   throw();
  virtual void f_f(float x) throw();
};

inline void Base::f(int x) throw()  // Overloaded non-virtuals
  { f_i(x); }
inline void Base::f(float x) throw()
  { f_f(x); }

void Base::f_i(int x) throw()       // Non-overloaded virtuals
  { cout << "Base::f(int)\n"; }
void Base::f_f(float x) throw()
  { cout << "Base::f(float)\n"; }

Base::~Base() throw()
  { }

class Derived : public Base {
public:
                                    // Derived classes never redefine f(int)
protected:
  virtual void f_i(int x) throw();  // Derived classes may override f_i(int)
};

void Derived::f_i(int x) throw()
  { cout << "Derived::f(int)\n"; }

void sample(Base& b, Derived& d)
{
  b.f(42);
  d.f(42);

  b.f(3.14f);
  d.f(3.14f);  // This is not hidden (good!)
}

int main()
{
  Derived d;
  sample(d, d);
}
