// Copyright (C) 1994-98 by Addison-Wesley Publishing Co., Inc.
//
// This code is provided for its instructional value.
// It has been tested with care, but it is not guaranteed
// for any particular purpose.  The publisher and authors
// do not offer any warranties or representations, nor do
// they accept any liabilities with respect to this code.

#include "cpp-faqs.hpp"

#include <memory>
#include <iostream>
#include <cstdlib>
#include <ctime>
using namespace std;

bool heads()    //a coin-toss: true with 50%-50% probability
{ return (rand() >> 4) % 2 ? true : false; }

class Noisy {
public:
  Noisy() throw();
 ~Noisy() throw();
  void g() throw(int);
  friend void h(Noisy& n) throw(int);
  Noisy& operator= (const Noisy&) throw();
  Noisy            (const Noisy&) throw();
};

Noisy::Noisy() throw()
  { cout << "Noisy::Noisy(); "; }
Noisy::~Noisy() throw()
  { cout << "Noisy::~Noisy(); "; }
void Noisy::g() throw(int)
  { cout << "Noisy::g(); ";  if (heads()) throw 5; }
void h(Noisy& n) throw(int)
  { cout << "h(Noisy&); "; if (heads()) throw 7; }

void usingManagedPointers() throw()
{
  cout << "using an auto_ptr<Noisy>: ";
  auto_ptr<Noisy> p(new Noisy());
  p->g();
  h(*p);
  cout << "didn't throw\n";
}  // The delete is automagic (no need to worry)

void usingRawPointers() throw()
{
  cout << "using a Noisy* pointer: ";
  Noisy* p = new Noisy();

  try {
    p->g();
    h(*p);
  }
  catch (...) {
    delete p;
    throw;
  }

  delete p;     // here the delete is explicit
  cout << "didn't throw\n";
}

int main()
{
  srand(time(NULL));   // Randomize the random number seed

  for (int i = 0; i < 10; ++i) {
    try {
      usingRawPointers();
      usingManagedPointers();
    } catch (int i) {
      cout << "caught " << i << '\n';
    }
    cout << '\n';
  }
}
